/*
 * $Id: network.h 367 2020-09-09 11:22:35Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Interface du filtre "Network provider" (interne).
 *
 *	Ce fichier dfinit l'interface du filtre (interne) "Network provider",
 *	qui est un pseudo-filtre de substitution au "Microsoft DVB-T Network Provider".
 *
 *	Il se place en tte de graphe, juste avant le tuner, et n'est utilis
 *	que dans le graphe du mode alternatif (le graphe normal utilisant effectivement
 *	le filtre "Microsoft DVB-T Network Provider").
 **/

#pragma once

#define NETWORK_FILTER_NAME "Network Provider PTvM"

/**
 * Pseudo-broche de sortie pour pseudo-filtre.
 * Cette broche de sortie n'a en ralit aucun flux  fournir.
 **/
class CPinBidon : public CSourceStream
{

public:
	CPinBidon(HRESULT *phr, CSource *pFilter);
	virtual ~CPinBidon();

	HRESULT CheckMediaType(const CMediaType *pmt);
	HRESULT GetMediaType(int iPosition, CMediaType *pMediaType);

	HRESULT FillBuffer(IMediaSample *pSamp);
	HRESULT DecideBufferSize(IMemAllocator * pAlloc,ALLOCATOR_PROPERTIES * ppropInputRequest);

};

/**
 * Pseudo-filtre ayant pour unique rle de se substituer au "Microsoft DVB-T Network Provider"
 * en dfinissant une broche de sortie  connecter au Tuner DVB.
 * Il y a des bribes de discussion sur Internet qui voquent cette approche,
 * par exemple ici :
 * http://forums.dvbowners.com/index.php?showtopic=3381&pid=30632
 * http://www.tech-archive.net/Archive/Development/microsoft.public.win32.programmer.directx.video/2006-04/msg00337.html
 **/
class DECLSPEC_UUID("1436DC04-A20F-46a7-B9FA-CF3C95A1E00C") CNetworkProvider : public CSource
{
private:
	CPinBidon * m_bidon;

public:
	CNetworkProvider(IUnknown *pUnk, HRESULT *phr);
	virtual ~CNetworkProvider();

	// IUnknown
	DECLARE_IUNKNOWN;
	//STDMETHODIMP	NonDelegatingQueryInterface(REFIID riid, void ** ppv);

	int GetPinCount(void);
	CBasePin *GetPin(int n);

	static LPCWSTR	szFilterName;
	static CUnknown * WINAPI CreateInstance(IUnknown *pUnk, HRESULT *phr);
};
