/*
 * $Id: atlbase_ptvm.h 306 2011-09-28 09:28:04Z gingko $
 * Copyright (C) 2008 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Substitution minimale aux modles de classes \p ATL
 *
 *	Ce fichier contient des susbtituts minimaux aux modles de classes
 *	(templates) \p ATL normalement inclus dans Visual Studio 2005
 *	dition professionnelle, mais non fournis dans Visual C++ 2005 Express.
 *
 *	Ces modles contiennent les fonctionnalits minimales ncessaires
 *	pour que le projet PouchinTV Mod puisse tre compil avec
 *	Visual C++ 2005 Express.
 *
 *	Les utilisateurs de Visual C++ 2005 Express qui dsireraient cependant
 *	avoir un support plus complet de ces modles de classes ATL peuvent
 *	tlcharger et installer le Windows Server 2003 SP1 Device Driver Kit
 *	(nom du fichier = \p 1830_usa_ddk.iso)  l'adresse suivante :
 *
 *	  http://www.microsoft.com/whdc/devtools/ddk/
 *
 *	Il convient ensuite d'ajouter les rpertoires suivants dans la liste des
 *	rpertoires de VC++ :
 *	- Fichiers include : \p \<RepDDK\>\\inc\\atl30
 *	- Fichiers bibliothques : \p \<RepDDK\>\\lib\\atl\\i386
 *
 *	\p \<RepDDK\> est le rpertoire d'installation du DDK
 *	(par dfaut \p C:\\WINDDK\\3790.1830).
 *
 *	\note Dernire minute (janvier 2010) : ce kit ne semble plus tre
 *	disponible chez Microsoft, mais on peut aisment le trouver sur Internet.
 *	Effectuer une recherche Google sur le nom du fichier.
 **/

#pragma once

// Le fichier "include_config.h" est le rsultat d'un test (appel depuis le script
// "include_config.bat") visant  savoir si les modles de classes AT sont disponibles.
// En effet, les modles de classes ATL ne sont pas disponibles dans l'environnement
// Visual C++ 2005 Express,  moins d'avoir install ces classes depuis une autre
// source. Voir commentaires dans le fichier atlbase_ptvm.h.
#include "include_config.h"

// Inclusions des en-ttes ATL dans Visual Studio, ou bien d'un en-tte s'y substituant dans les ditions VC++ Express
#if EXIST_ATLBASE_H

	// Si on a une dition complte de Visual Studio 2005 ou 2008 (donc pas une dition Express),
	// on utilise les en-ttes ATL qui sont fournis avec.

	#pragma warning(push)
	#pragma warning(disable:4995)	// Quelques inconsistances de Microsoft dans les en-ttes ATL

	#include <atlbase.h>
	#include <atlmem.h>

	#pragma warning(pop)

#else		// #if (choix de la source pour les classes ATL)


	// Sinon, on utilise la version "light" fournie ici

	#include <unknwn.h>	// Requis, particulirement avec VCExpress

// ================================================================================================
template <class IU>
	class CComPtr
{
public:
	IU * p;

	CComPtr() : p(NULL) {
		}
	CComPtr(IU* q) : p(q) {
		if (p) p->AddRef(); }
	CComPtr(const CComPtr<IU> & r) : p(r.p) {
		if (p) p->AddRef(); }
	IU * operator -> () const {
		return p; }
	bool operator ! () const {
		return p==NULL; }
	IU** operator & () {
		return &p; }
	operator IU * () const {
		return p; }
	void Release() {
		IU* t=p; if (t) {p=NULL; t->Release(); } }
	void Attach(IU* q) throw() {
		if (p) p->Release(); p = q; }
	IU* Detach() {
		IU* pt = p; p = NULL; return pt; }
	IU* operator = (IU* q) {
		Release(); p=q; p->AddRef(); return p; }
	IU* operator = (const CComPtr<IU> & r) {
		Release(); p=r.p; p->AddRef(); return *this; }
	template <typename IU2> IU* operator=(const CComPtr<IU2> & r) {
		Release(); p=dynamic_cast<IU*>(r.p); p->AddRef(); return *this; }
	template <class QI> HRESULT QueryInterface(QI** pp) const {
		return p->QueryInterface(__uuidof(QI), (void**)pp); }
	HRESULT CoCreateInstance(const GUID & g, IUnknown* u=NULL, unsigned long c=CLSCTX_ALL) {
		return ::CoCreateInstance(g, u, c, __uuidof(IU), (void**)&p); }
	~CComPtr() {
		Release(); }
};

template <class IU, const IID * I = &__uuidof(IU)>
	class CComQIPtr : public CComPtr<IU>
{
public:
	CComQIPtr(IUnknown* q) {
		if (q != NULL) q->QueryInterface(*I, (void **)&p); }
};

template <class PT>
	class CAdapt
{
public:
	CAdapt() {
		}
	CAdapt(const PT & t2) : m_T( t2 ) {
		}
	CAdapt(const CAdapt& ca2) : m_T( ca2.m_T ) {
		}

	CAdapt& operator=(const PT & t2) {
			m_T = t2; return *this; }
	bool operator<(const PT & t2) const {
			return m_T < t2; }
	bool operator==(const PT & t2) const {
			return m_T == t2; }
	operator PT & () {
			return m_T; }

	operator const PT & () const {
			return m_T; }

	PT m_T;
};

class CComVariant : public VARIANT
{
public:
	CComVariant() {
		VariantInit(this); }
	CComVariant(long val) {
		vt = VT_I4; lVal = val; }
	~CComVariant() {
		VariantClear(this); }
};

template <class CP>
	class CComHeapPtr
{
public:
	CP* m_pData;

	CComHeapPtr() : m_pData(NULL) {
		}
	operator CP * () {
		return m_pData; }
	operator const CP * () const {
		return m_pData; }
	bool AllocateBytes(size_t siz) {
		m_pData = static_cast<CP *>(CoTaskMemAlloc(siz));
		return (m_pData != NULL); }
	void Free() {
		CP* t=m_pData; if (t) {m_pData=NULL; CoTaskMemFree(t); } }
	CP** operator & () {
		return &m_pData; }
	~CComHeapPtr() {
		Free(); }
};

class CComBSTR
{
public:
	BSTR m_str;

	CComBSTR() : m_str(NULL) {
		}
	CComBSTR(LPCSTR pSrc) {
		if (pSrc != NULL) {
			int taille = MultiByteToWideChar(CP_ACP, 0, pSrc, -1, NULL, NULL);
			m_str = ::SysAllocStringLen(NULL, taille);
			if (m_str != NULL) {
				int taille2 = MultiByteToWideChar(CP_ACP, 0, pSrc, -1, m_str, taille);
				if (taille2 != taille)
					::SysFreeString(m_str);
			}
		} else {
			m_str = NULL;
		}
	}

	operator BSTR() const {
		return m_str; }
	BSTR* operator&() {
		return &m_str; }

};

// ================================================================================================

#endif		// #if (choix de la source pour les classes ATL)
