/*
 * $Id: chanutils.h 348 2012-07-01 20:27:42Z gingko $
 * Copyright (C) 2008 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions de structures pour la sauvegarde des flux TS associs  une chane.
 *
 *	Ce fichier dfinit des structures et fonctions utilitaires lies  l'identification des
 *	flux TS utiliss par les chanes TV.
 *
 *	Ces lments sont utiliss, d'une part pour la gestion des structures de sauvegarde des
 *	informations lies  ces chanes, et d'autre part pour la slection des pistes dans les
 *	enregistrements programms.
 **/

#pragma once

#include "utils.h"

#define STREAM_PSEUDO_CLE 0xffffffff	//!< Pseudo cl pour l'enregistrement du multiplex

/// numration indiquant le types de la piste vido
enum VideoStreamType : UINT8
{
	vst_Unknown = 0xff,		//!< Type vido inconnu
	vst_MPEG2 = 0,			//!< Vido MPEG2
	vst_H264,				//!< Vido H264 (SD ou HD)
	vst_MPEG2_encrypted,	//!< Vido MPEG2 crypte (hypothtique, pour tre exhaustif)
	vst_H264_encrypted		//!< Vido H264 crypte (SD ou HD)
};

/// numration indiquant les types des pistes "son" et "autres"
enum TrackStreamType : UINT8
{
	tst_MPEG2,				//!<   0 = MPEG2 Audio Layer II
	tst_AC3,				//!<   1 = AC3
	tst_EAC3,				//!<   2 = Enhanced AC3
	tst_AC3_hc,				//!<   3 = AC3 Home Cinema (plus de 2 canaux, style 5.1)
	tst_EAC3_hc,			//!<   4 = Enhanced AC3 Home Cinema
	tst_AAC,				//!<   5 = AAC
	tst_AAC_hc,				//!<   6 = AAC Home Cinema
	tst_Audio_END,			//!< (limite suprieure des codes audio)
	//
	// Autres formats :
	tst_Subtitling = 0x80,	//!< 128 = Sous-titrage
	tst_Teletext,			//!< 129 = Tltexte
	tst_Other_END,			//!< (limite suprieure des codes "autres")
	//
	tst_Other_BEG = tst_Subtitling,	//!< (limite infrieure des codes "autres")
	//
	tst_Unknown = 0xff		//!< 255 = Inconnu
};

/// numration indiquant les prfrences audio de l'utilisateur
enum AudioMode {
	am_None,				//!< Aucune prfrence
	am_Mpeg_1,				//!< Utilisation de la premire piste MPEG2 disponible
	am_Mpeg_2,				//!< Utilisation de la seconde piste MPEG2 disponible
	am_AC3,					//!< Utilisation de la premire piste AC3 ou E-AC3
	am_AC3_non_HC,			//!< Utilisation de la premire piste AC3 ou E-AC3 non HC
	am_AC3_HC,				//!< Utilisation de la premire piste AC3 ou EAC, +HC si disponible
	am_AAC,					//!< Utilisation de la premire piste AAC
	am_AAC_non_HC,			//!< Utilisation de la premire piste AA non HC
	am_AAC_HC,				//!< Utilisation de la premire piste AAC, +HC si disponible
	am_VO					//!< Utilisation de la premire piste non francophone disponible
};

/// Nombre maximal de pistes "son" + "autre" par chane
#define NB_MAX_PISTES 32

/// Description d'une piste ("son" ou "autre")
struct une_piste {
	UINT16			pid;
	TrackStreamType type;	// si son : voir TrackStreamType; tst_Unknown (rserv) si autres
	char			lang[8];

	/// Constructeur par dfaut
	une_piste() :
		pid(0),
		type(tst_Unknown)
	{
		// Mise  zro du tableau de caractres
		ZeroMemory(lang, sizeof(lang));
	}

	/// Constructeur copie
	une_piste(const une_piste & piste) :
		pid(piste.pid),
		type(piste.type)
	{
		strcpy_T(lang, piste.lang);
	}

	/// Comparaison pour galit
	bool operator == (const une_piste & up) const;

	/// Comparaison pour diffrence
	bool operator != (const une_piste & up) const {
			return ! operator == (up); }

	/**
	 * Obtenir une chane reprsentant la dfinition de la fiche, pour affichage ou journalisation
	 * \param[in,out]	pszBuf	Pointeur sur une chane recevant le rsultat
	 * \param[in]		nBufSiz	Longueur de cette chane
	 **/
	void getAsLogString(LPTSTR pszBuf, UINT32 nBufSiz) const;

	/// Tester s'il s'agit d'une piste MPEG2
	bool isMPEG2() const {
		return type == tst_MPEG2; }

	/// Tester s'il s'agit d'une piste AC3
	bool isAC3() const {
		return type >= tst_AC3 && type <= tst_EAC3_hc; }

	/// Tester s'il s'agit d'une piste AC3 HC (plus de deux canaux)
	bool isAC3_HC() const {
		return type == tst_AC3_hc || type == tst_EAC3_hc; }

	/// Tester s'il s'agit d'une piste AC3 non HC (au plus deux canaux)
	bool isAC3_non_HC() const {
		return type == tst_AC3 || type == tst_EAC3; }

	/// Tester s'il s'agit d'une piste en V.O. (langue diffrente de franais)
	bool isVO() const {
		return lang[0]!=0 && strcmp(lang, "fra")!=0 && strcmp(lang, "fre")!=0; }
};

/// Table des pistes associes  une chane (son en premier, autres ensuite)
struct liste_pistes {
	UINT16			nbr;				//!< Nombre de pistes son
	UINT16			nbr_tot;			//!< Nombre de pistes total
	une_piste		tbl[NB_MAX_PISTES];	//!< Table des pistes

	/// Constructeur par dfaut
	liste_pistes() :
		nbr(0),
		nbr_tot(0)
	{
	}

	/**
	 * Obtenir l'index de la premire piste "son" ou "autre"
	 * \param[in]	bOther	\p false pour "son", \p true pour "autre"
	 **/
	UINT16	getFirst(bool bOther) const {
		return bOther ? nbr : 0; }

	/**
	 * Obtenir le nombre de pistes de type "son" ou "autre"
	 * \param[in]	bOther	\p false pour "son", \p true pour "autre"
	 **/
	UINT16	getCount(bool bOther) const {
		return bOther ? nbr_tot-nbr : nbr; }

	/**
	 * Ajout d'une piste  la liste, avec dtermination de la cattorie ("son" ou "autre")
	 * selon le type de la piste
	 * \param[in]	piste	Piste  ajouter
	 * \return		Index de la piste qui vient d'tre ajout
	 * \note	L'insertion d'une piste "son" va dcaler les index des pistes "autre" qui
	 *			pourraient exister pralablement
	 **/
	int ajoute(const une_piste & piste);

	/**
	 * Recherche de la Nme piste audio correspondant  un critre donn
	 * \param[in]	pfn		Pointeur sur une fonction membre de test de critre
	 * \param[in]	nCount	Valeur de "N", numro de piste recherch
	 * \return		Index de la piste trouve, ou -1 si aucune
	 **/
	int	cherche_piste(bool (une_piste::*pfn)() const, int nCount = 1) const;

	/**
	 * Trouver la piste audio la mieux adapte au choix de l'utilisateur
	 * \param[in]	eAudio	Code de prfrence de l'utilisateur
	 * \return		Index de la piste audio slectionne, ou bien -1 en cas d'chec
	 **/
	int selection_son(AudioMode eAudio) const;

	/// Comparaison pour galit
	bool operator == (const liste_pistes & lp) const;

	/// Comparaison pour diffrence
	bool operator != (const liste_pistes & lp) const {
		return ! operator == (lp); }
};

/// Structure de dfinition des pistes d'une chane
struct ChainePMT : public liste_pistes {
	UINT16			pmt_pid;	//!< PID de la table PMT
	UINT16			video_pid;	//!< PID du flux vido
	VideoStreamType	video_type;	//!< Type du flux vido
	UINT16			pcr_pid;	/*!< PID contenant le "Program Clock Reference" (= en principe
									 au PID d'une autre piste existante, vido, audio ...)
									 \todo Ncessaire ? */
	/// Constructeur par dfaut
	ChainePMT();

	/**
	 * Rechercher si un "pid" donn fait partie de la dfinition des flux TS d'une chane
	 * \param[in]	pid	Numro de "pid" recherch
	 * \return		\p true si le "pid" est dfini dans la chane
	 **/
	bool cherche_pid(UINT16 pid) const;

	/// Obtenir une chane de caractres reprsentant le codage vido utilise
	/// pour la chane, pour affichage
	LPCTSTR norme() const;

	/// Comparaison pour galit
	bool operator == (const ChainePMT & cpmt) const;

	/// Comparaison pour diffrence
	bool operator != (const ChainePMT & cpmt) const {
			return ! operator == (cpmt); }

	/// Dterminer si une chane est crypte ou pas
	bool isEncrypted() const {
			return video_type>=vst_MPEG2_encrypted; }
};

// Dfinition de l'ensemble des identifiants d'une chane :
struct ChaineIDs
{
	UINT16		ONID;		//!< Original Network ID (spcifique au rseau)
	UINT16		TSID;		//!< Transport Stream ID (spcifique au multiplex)
	UINT16		SID;		//!< Service ID (spcifique  la chane)
	UINT16		canal_no;	//!< Numro du canal (note : la mme chane peut tre reue sur
							//!< plusieurs canaux si on se trouve  porte de plusieurs metteurs)
	// La taille totale doit tre un multiple de 64 bits : aucun alignement, tant en mode x86
	// que x64, ne doit induire de "trous" dans lesquels pourraient se trouver des contenus
	// imprvisibles qui fausseraient le calcul de la cl.

	ChaineIDs() :
		ONID(0),
		TSID(0),
		SID(0),
		canal_no(0)
	{
	}

	ChaineIDs(UINT16 onid, UINT16 tsid, UINT16 sid, BYTE can_no=0) :
		ONID(onid),
		TSID(tsid),
		SID(sid),
		canal_no(can_no)
	{
	}

	/// Fonction retournant \p true si les identifiants \p sCIds dsignent une chane identique
	///  ceux correspondants  la chane \p this.
	/// Toute valeur nulle (sauf SID) vaut identit dans la comparaison.
	bool EquIds(const ChaineIDs &sCIds) const;

	/// Gnration d'une cl sur 32 bits visant  identifier la chane d'une faon (qu'on
	/// espre raisonnablement) unique
	UINT32 cle() const;
};

/// Identifiants de chane associs  un numro dfini par le rseau
struct ChaineIdsNIT : public ChaineIDs
{
	UINT16		nNumeroNit;	//!< Numro de chane original dfini par le rseau

	ChaineIdsNIT() :
		nNumeroNit(0)
	{
	}
};
