/*
 * $Id: dshow_inc.h 297 2011-08-03 21:07:48Z gingko $
 * Copyright (C) 2009 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Fichier d'inclusion global pour l'utilisation de \p DirectShow
 *
 *	Ce fichier regroupe en un seul point l'ensemble des directives ncessaires 
 *	l'utilisation de DirectShow, ainsi que les amnagements utiliss pour contourner
 *	certains problmes rencontrs.
 **/

#pragma once

// ====================================================================================
// Inclusions spcifiques  l'utilisation de DirectShow
// ====================================================================================

#include <iostream>
#include <dshow.h>

#include <ks.h>
#include <ksmedia.h>
#include <bdatypes.h>
#include <bdamedia.h>

#include <d3d9.h>
#include <vmr9.h>
#include <evr9.h>
#include <Dvdmedia.h>

// Les dclarations suivantes sont rendues ncessaires par la suppression
// du fichier "qedit.h" dans la version 7 du Windows SDK :

// Voir http://social.msdn.microsoft.com/Forums/en-US/windowsdirectshowdevelopment/thread/2ab5c212-5824-419d-b5d9-7f5db82f57cd

/// L'objet \p CLSID_NullRenderer n'est plus dclar nulle part
class DECLSPEC_UUID("C1F400A4-3F08-11d3-9F0B-006008039E37") NullRenderer;	// Accs par __uiidof(NullRenderer)

/// L'objet \p ISampleGrabberCB n'est plus dclar nulle part
MIDL_INTERFACE("0579154A-2B53-4994-B0D0-E773148EFF85")
	ISampleGrabberCB : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE SampleCB(double SampleTime, IMediaSample *pSample) = 0;
	virtual HRESULT STDMETHODCALLTYPE BufferCB(double SampleTime, BYTE *pBuffer, long BufferLen) = 0;
};

class DECLSPEC_UUID("C1F400A0-3F08-11d3-9F0B-006008039E37") SampleGrabber;

/// L'objet \p ISampleGrabber n'est plus dclar nulle part
MIDL_INTERFACE("6B652FFF-11FE-4fce-92AD-0266B5D7C78F")
	ISampleGrabber : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE SetOneShot(BOOL OneShot) = 0;
	virtual HRESULT STDMETHODCALLTYPE SetMediaType(const AM_MEDIA_TYPE *pType) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetConnectedMediaType(AM_MEDIA_TYPE *pType) = 0;
	virtual HRESULT STDMETHODCALLTYPE SetBufferSamples(BOOL BufferThem) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetCurrentBuffer(long *pBufferSize, long *pBuffer) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetCurrentSample(IMediaSample **ppSample) = 0;
	virtual HRESULT STDMETHODCALLTYPE SetCallback(ISampleGrabberCB *pCallback, long WhichMethodToCallback) = 0;
};

// Autres dclarations qui ne sont plus incluses dans le SDK Windows
#define CLSID_SampleGrabber		__uuidof(SampleGrabber)
#define CLSID_NullRenderer		__uuidof(NullRenderer)
#define IID_ISampleGrabberCB	__uuidof(ISampleGrabberCB)
