/*
 * $Id: mediadefs.h 264 2010-11-01 22:49:12Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions unifies de mdias
 *
 *	Ce fichier dfinit les types de donnes ncessaires pour dfinir l'ensemble des types
 *  de mdias audio et vido, ainsi que les pistes et tables spciales qu'on peut rencontrer
 *	dans un flux Multimdia MPEG, TS, PS ou autre.
 *
 *	Il inclut galement quelques dfinitions de fonctions de conversion et de reprsentation
 *	de ces types de donnes.
 **/

#pragma once

// Liste des types de flux lmentaires (ES = Elementary Stream) qui peuvent tre rencontrs :
enum MediaType : UINT8 {
	mty_Undef				= 0,
	mty_Unknown				= 1,
	//
	mty_SFN					= 0x10,
	//
	mty_SI_pat				= 0x20,
	mty_SI_cat				= 0x21,
	mty_SI_nit				= 0x22,
	mty_SI_sdt				= 0x23,
	mty_SI_eit				= 0x24,
	//
	mty_SI_pmt				= 0x28,
	//
	mty_SI_private			= 0x30,
	mty_SI_unknown			= 0x3f,
	//
	mty_Video				= 0x40,	//!< Vido Mpeg, gnrique
	mty_Video_Mpeg1			= 0x42,	//!< Vido Mpeg 1
	mty_Video_Mpeg2			= 0x44,	//!< Vido Mpeg 2
	mty_Video_Mpeg2_crypt	= 0x45,	//!< Vido Mpeg 2 crypt
	mty_Video_H264			= 0x48,	//!< Vido H264
	mty_Video_H264_crypt	= 0x49,	//!< Vido H264 crypt
	//
	mty_Audio				= 0x80,	//!< Audio Mpeg, gnrique
	mty_Audio_Mpeg1			= 0x81,	//!< Audio Mpeg 1, gnrique
	mty_Audio_Mpeg2			= 0x82,	//!< Audio Mpeg 2, gnrique
	mty_Audio_Mpeg1_L1		= 0x83,	//!< Audio Mpeg 1/2 layer 1
	mty_Audio_Mpeg1_L2		= 0x84,	//!< Audio Mpeg 1/2 layer 2
	mty_Audio_Mpeg1_L3		= 0x85,	//!< Audio Mpeg 1 layer 3
	mty_Audio_Mpeg2_L3		= 0x86,	//!< Audio Mpeg 2 layer 3
	mty_Audio_Aac			= 0x90,	//!< Audio AAC, gnrique
	mty_Audio_Ac3			= 0xa0,	//!< Audio AC3
	mty_Audio_Ac3_hc		= 0xa1,	//!< Audio AC3 Home Cinema (plus de 2 canaux, style 5.1)
	mty_Audio_Ac3_atsc		= 0xa2,
	mty_Audio_EAc3			= 0xa8,	//!< Audio E-AC3
	mty_Audio_EAc3_hc		= 0xa9,	//!< Audio E-AC3 Home Cinema
	mty_Audio_EAc3_atsc		= 0xaa,
	//
	mty_Private				= 0xc0,
	mty_Subtitling			= 0xc1,
	mty_Teletext			= 0xc2,
	mty_Other				= 0xcf
};

// Aliases :
#define	mty_SI_FIRST		mty_SI_pat
#define	mty_SI_LAST			mty_SI_unknown
#define	mty_Video_FIRST		mty_Video
#define	mty_Video_LAST		mty_Video_H264
#define	mty_Audio_FIRST		mty_Audio
#define	mty_Audio_LAST		mty_Audio_EAc3_atsc
#define	mty_Other_FIRST		mty_Private
#define	mty_Other_LAST		mty_Other
#define	mty_FIRST			mty_Video_FIRST
#define	mty_LAST			mty_Other_LAST

/// Conversion des types de mdia MPEG en types de mdia locaux
MediaType MpegType2MediaType(StreamType eType);

/// Table des noms de flux
extern const AssocElement aMediaTypeTable[];
extern const AssocElement aAudioTypeTable[];

// ----------------------------------------------------------------------------

/// Interface de base pour captures et conversions gnriques
class MpegInput
{
public:
	virtual ~MpegInput() {
				}
};

typedef shared_ptr<MpegInput>	MpegInputPtr;

// ----------------------------------------------------------------------------

/// Interface de base pour captures et conversions depuis une source Mpeg2 TS
class Mpeg_TS_Input : public MpegInput
{
public:
	/// Envoi d'un paquet TS
	virtual DWORD	traite_paquet(const TS_packet & tsp) = 0;
};

typedef shared_ptr<Mpeg_TS_Input>	Mpeg_TS_InpPtr;

// ----------------------------------------------------------------------------

