/*
 * $Id: sbuffer.cpp 348 2012-07-01 20:27:42Z gingko $
 * Copyright (C) 2008 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Gestion d'un tampon de donnes  croissance automatique (implmentations)
 *
 *	Ce fichier implmente les classes utilises pour la construction d'un tampon
 *	dont la taille crot automatiquement en cas de dpassement de longueur.
 *	Ce tampon est utilis pour la conversion interne TS -> PS.
 **/

#include "stdafx.h"
#include "sbuffer.h"

// ----------------------------------------------------------------------------

/// Envoi et vidange du tampon
void StreamBuffer::flush() throw(...)
{
	if (get().siz > 0) {
		sample_end(get().end());
		prepare(get());
		send(get());
		discard();
	}
}

void StreamBuffer::put_data(MemRng sMm, bool start) throw(...)
{
	if (start) {
		flush();
	}

	if (sMm.siz > 0) {
		UINT nPrevSize = get().siz;

		// Copie des donnes source dans le tampon
		append(sMm);
		if (test(nPrevSize)) {
			prepare(get());
			send(get());
		}
	}
}

// change des donnes avec celles d'un autre tampon de mme type :
void StreamBuffer::swap(StreamBuffer & b2swap)
{
	auto_vector::swap(b2swap);
}

// ----------------------------------------------------------------------------
