/*
 * $Id: stdafx.h 305 2011-09-25 22:41:16Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Fichier d'inclusion gnral pour les en-ttes prcompils.
 *
 *	Ce fichier regroupe toutes les directives d'inclusions de fichiers d'en-ttes \p (.h) qui
 *	doivent tre incluse dans le fichier d'en-tte prcompils.
 *
 *	Il est associ au fichier \p stdafx.cpp, en principe vide, mais servant de dclencheur
 *	 la gnration de ces en-ttes par le fait qu'il est le seul  tre compil (selon le
 *	fichier projet) avec l'option \p /Yc (Crer un en-tte prcompil), alors que les autres
 *	fichiers sont  compiler avec l'option \p /Yu (Utiliser un en-tte prcompil).
 *
 *	Le fichier d'en-tte prcompil a l'extension \p .pch.
 **/

#pragma once

// -----------------------------------------

/// Dfinition Visual C++ permettant de rduire la taille des fichiers
/// d'en-tte Win32 en excluant certaines API peu courantes
#define WIN32_LEAN_AND_MEAN

/// Palliatif pour faire croire que le fichier <d3drm.h> est dj inclus, et ainsi s'viter
/// des problmes avec certaines versions du DirectX SDK.
#define __D3DRM_H__

/// Certains dfinitions multimdia requirent ceci pour exister
#define _INC_MMREG

// -----------------------------------------

/**
 * Dfinition de la version minimale de Windows prise en charge (version avance)
 * \see http://blogs.msdn.com/oldnewthing/archive/2007/04/11/2079137.aspx
 **/
#define NTDDI_VERSION	NTDDI_WINXPSP2		// Windows XP Service Pack 2

/**
 * Dfinition de la version minimale de Windows prise en charge (version hrite)
 * \see http://blogs.msdn.com/oldnewthing/archive/2007/04/11/2079137.aspx
 **/
#define	_WIN32_WINNT	_WIN32_WINNT_WINXP	// Windows XP

/// vite des warnings, y compris dans certains en-ttes Microsoft inclus
#define _CRT_SECURE_NO_WARNINGS

// -----------------------------------------

// Test de la version de Visual Studio, pour vrifier
// si certaines fonctionnalits sont disponibles ou pas
#if _MSC_VER >= 1600	// ->  partir de Visual Studio 2010
	#define TYPE_INFERENCE_AVAILABLE	1
#else					// -> versions plus anciennes
	#define TYPE_INFERENCE_AVAILABLE	0
#endif

// -----------------------------------------

#include <windows.h>
#include <tchar.h>
#include <commctrl.h>

#include <vector>
#include <string>
#include <algorithm>

using namespace std;

// En-ttes Pouchin TV Mod Frquemment requis

#include "atlbase_ptvm.h"		// En-ttes ATL ou substitution
#include "shared_ptr_ptvm.h"	// Template "shared_ptr" ou substitution

#include "xmlutils.h"
#include "utils.h"
#include "rctutils.h"
#include "memrng.h"
#include "mpeg2defs.h"
#include "mediadefs.h"
#include "console_select.h"
#include "filedlg.h"
