/*
 * $Id: xmlutils.cpp 304 2011-09-25 08:04:03Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentations des fonctions utilitaires lies  \p TinyXML
 *
 *	Ce fichier implmente les fonctions utilitaires d'assistance
 *	 l'usage de la bibliothque \p TinyXML.
 **/

#include "stdafx.h"
#include "xmlutils.h"

/**
 * Obtenir le contenu d'un lment XML, ou bien une chane vide si
 * cet lment n'existe pas ou s'il n'est pas du texte
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[in]	pszTag	lment recherch
 * \param[out]	ppChild	Pointeur sur une adresse recevant le noeud enfant si non NULL
 * \return		Le texte inclus dans  l'lment, ou bien une chane vide si l'lment
 *				n'existe pas ou bien n'est pas une chane.
 * \note		Si plusieurs lments de mme nom existent, seul le premier est retourn
 **/
LPCSTR GetXmlContent(const TiXmlElement * pXmlElm, LPCSTR pszTag, const TiXmlElement ** ppChild)
{
	const TiXmlElement * pXmlTmp = pXmlElm->FirstChildElement(pszTag);

	if (ppChild)
		*ppChild = pXmlTmp;
	if (pXmlTmp)
		return pXmlTmp->GetText();
	return "";
}

/**
 * Obtenir le contenu d'un lment XML, ou bien une chane vide si
 * cet lment n'existe pas ou s'il n'est pas du texte.
 * Le nom de l'lment lui-mme fait l'objet d'un formatage.
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[out]	ppChild	Pointeur sur une adresse recevant le noeud enfant si non NULL
 * \param[in]	pszFmt	Chane de formatage
 * \param		...		Arguments de la chane de formatage
 * \return		Le texte inclus dans  l'lment, ou bien une chane vide si l'lment
 *				n'existe pas ou bien n'est pas une chane.
 * \note		Si plusieurs lments de mme nom existent, seul le premier est retourn
 **/
LPCSTR GetXmlContentF(const TiXmlElement * pXmlElm, const TiXmlElement ** ppChild, LPCSTR pszFmt, ...)
{
	CHAR	szTmp[16];
	va_list	args;

	va_start(args, pszFmt);
	vsprintf_s(szTmp, pszFmt, args);
	va_end(args);
	return GetXmlContent(pXmlElm, szTmp, ppChild);
}

/**
 * Dfinir le contenu d'un lment XML
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[in]	pszTag	lment recherch
 * \param[in]	pszTxt	Texte  y inclure
 **/
TiXmlElement * SetXmlContent(TiXmlElement * pXmlElm, LPCSTR pszTag, LPCSTR pszTxt)
{
	TiXmlElement *	pXmlChld(new TiXmlElement(pszTag));

	if (pXmlChld) {
		TiXmlText * pXmlTxt(new TiXmlText(pszTxt));

		if (pXmlTxt) {
			pXmlChld->LinkEndChild(pXmlTxt);
			pXmlElm->LinkEndChild(pXmlChld);
		} else
			delete pXmlChld;
	}
	return pXmlChld;
}

/**
 * Dfinir le contenu d'un lment XML
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[in]	pszFmt	Chane de formatage
 * \param[in]	pszTxt	Texte  y inclure
 * \param		...		Arguments de la chane de formatage
 **/
TiXmlElement * SetXmlContentF(TiXmlElement * pXmlElm, LPCSTR pszFmt, LPCSTR pszTxt, ...)
{
	CHAR	szTmp[16];
	va_list	args;

	va_start(args, pszTxt);
	vsprintf_s(szTmp, pszFmt, args);
	va_end(args);
	return SetXmlContent(pXmlElm, szTmp, pszTxt);
}

