/*
 * $Id: xmlutils.h 304 2011-09-25 08:04:03Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Fonctions et macros utilitaires lie  \p TinyXML (dfinitions)
 *
 *	Ce fichier contient des fonctions et macros utilitaires d'assistance
 *	 l'usage de la bibliothque \p TinyXML.
 *
 *	\note L'inclusion de ce seul fichier suffit  inclure galement
 *	les fichiers de dfinition \p TinyXML.
 **/

#pragma once

/**
 * Ceci permet d'activer ou de dsactiver certaines fonctionnalits
 * selon que cet en-tte a t inclus ou pas.
 **/
#define _XMLUTILS_H_INCLUDED_

#define TIXML_USE_STL	// Utilisation de variables chanes format STL dans TinyXml

#include "tinyxml.h"

/// Macro d'assistance  l'numration de nuds enfants d'un nud TiXML
#define ITERATE_XML_CHILDREN(parent, child) \
	const TiXmlNode * child = (parent)->FirstChild(); \
	child != NULL; \
	child = child->NextSibling()

/// Macro d'assistance  l'numration d'lments enfants d'un nud TiXML
#define ITERATE_XML_ELEMENTS(parent, child) \
	const TiXmlElement * child = (parent)->FirstChildElement(); \
	child != NULL; \
	child = child->NextSiblingElement()

/// Macro d'assistance  l'numration d'lments enfants nomms d'un nud TiXML
#define ITERATE_XML_ELEMENTS_NAMED(parent, child, name) \
	const TiXmlElement * child = (parent)->FirstChildElement(name); \
	child != NULL; \
	child = child->NextSiblingElement(name)

// ----------------------------------------------------------------------------

/**
 * Obtenir le contenu d'un lment XML, ou bien une chane vide si
 * cet lment n'existe pas ou s'il n'est pas du texte
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[in]	pszTag	lment recherch
 * \param[out]	ppChild	Pointeur sur une adresse recevant le noeud enfant si non NULL
 * \return		Le texte inclus dans  l'lment, ou bien une chane vide si l'lment
 *				n'existe pas ou bien n'est pas une chane.
 * \note		Si plusieurs lments de mme nom existent, seul le premier est retourn
 **/
LPCSTR GetXmlContent(const TiXmlElement * pXmlElm, LPCSTR pszTag, const TiXmlElement ** ppChild = NULL);

/**
 * Obtenir le contenu d'un lment XML, ou bien une chane vide si
 * cet lment n'existe pas ou s'il n'est pas du texte.
 * Le nom de l'lment lui-mme fait l'objet d'un formatage.
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[out]	ppChild	Pointeur sur une adresse recevant le noeud enfant si non NULL
 * \param[in]	pszFmt	Chane de formatage
 * \param		...		Arguments de la chane de formatage
 * \return		Le texte inclus dans  l'lment, ou bien une chane vide si l'lment
 *				n'existe pas ou bien n'est pas une chane.
 * \note		Si plusieurs lments de mme nom existent, seul le premier est retourn
 **/
LPCSTR GetXmlContentF(const TiXmlElement * pXmlElm, const TiXmlElement ** ppChild, LPCSTR pszFmt, ...);

/**
 * Dfinir le contenu d'un lment XML
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[in]	pszTag	lment recherch
 * \param[in]	pszTxt	Texte  y inclure
 **/
TiXmlElement * SetXmlContent(TiXmlElement * pXmlElm, LPCSTR pszTag, LPCSTR pszTxt);

/**
 * Dfinir le contenu d'un lment XML
 * \param[in]	pXmlElm	Noeud XML parent
 * \param[in]	pszFmt	Chane de formatage
 * \param[in]	pszTxt	Texte  y inclure
 * \param		...		Arguments de la chane de formatage
 **/
TiXmlElement * SetXmlContentF(TiXmlElement * pXmlElm, LPCSTR pszFmt, LPCSTR pszTxt, ...);
