/*
 * $Id: parse.h 306 2011-09-28 09:28:04Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions lies  l'exploration des tables SI des flux Mpeg2 TS
 *
 *	Ce fichier inclut les dfinitions ncessaires  pour permettre l'exploration
 *	des tables SI des flux Mpeg2 TS.
 **/

#pragma once

#include "chanutils.h"

/**
 * Analyse d'une table PAT (Program Association Table) reue dans le flux TS
 * \param[in]	pat			Table  analyser
 * \param[in]	nNoCanal	Numro du canal dans lequel se trouve la table
 **/
void parse_pat(const SI_PAT & pat, BYTE nNoCanal);

/**
 * Analyse d'une table PMT reue dans le flux TS, en vue de dterminer le format du flux vido
 * \param[in]	pmt			Table  analyser
 * \param[in]	bMicrosoft	\p true si la table a t obtenue, transforme, depuis une fonction
 *							Microsoft, \p false si elle l'a t depuis un flux natif
 * \param[in]	lpmt_e		Rsultat de l'analyse
 * \note Les fonctions Microsoft transforment le contenu d'une partie de la table, en inversant
 * l'ordre des octets de certains lments de l'en-tte (big endian -> little endian)
 **/
void parse_pmt(const SI_PMT & pmt, bool bMicrosoft, ChainePMT & lpmt_e);

/**
 * Analyse d'une table SDT reue dans le flux TS
 * \param[in]	sdt			Table  analyser
 * \param[in]	nNoCanal	Numro du canal dans lequel se trouve la table
 * \param[in]	khz			Frquence du canal dans lequel se trouve la table
 **/
void parse_sdt(const SI_SDT & sdt, BYTE nNoCanal, INT32 khz);

/// Contient les informations rcupres dans la NIT pour dfinir les numros des chanes.
class CInfoNIT : public vector<ChaineIdsNIT>
{
public:
	/// Nom du rseau (utilis dans le log)
	tstring					strNetworkName;

	/**
	 * Recherche dans la table si on a des informations
	 * concernant la chane identifie par \p sCIds.
	 * \param[in]	sCIds	Identifiants de la chane recherche
	 * \return		Pointeur sur le descripteur existant, ou bien \p NULL si pas trouv
	 **/
	const ChaineIdsNIT *	Cherche(const ChaineIDs & sCIds) const;
};

/**
 * Analyse d'une table NIT reue dans le flux TS
 * \param[in]	nit			Table  analyser
 * \param[in]	cInfoNIT	Donnes rcupres
 **/
void parse_nit(const SI_NIT & nit, CInfoNIT & cInfoNIT);
