/*
 * $Id: scan.h 306 2011-09-28 09:28:04Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions lies  la recherche de chanes
 *
 *	Ce fichier inclut les dfinitions ncessaires  la fonctionnalit
 *	de recherche de chanes (hors interface utilisateur).
 **/

#pragma once

#include "parse.h"
#include "channels.h"

/// tat de la recherche de chanes
enum ScanState {
	ss_idle,		//!< Inactif
	ss_cancelling,	//!< En cours d'annulation
	ss_stopping,	//!< En cours d'arrt
	ss_running		//!< Recherche en cours
};

/// Codes de notification passs dans \p LOBYTE(wParam) avec le message \p WM_APP_SCAN_NOTIFY
enum ScanNotifications {
	sn_init,		//!< Initialisation (\p lParam = nombre d'tapes, \p HIWORD(wParam) = numro rptition)
	sn_info_freq,	//!< Message frquence en cours de recherche
					//!< \p lParam = frquence en cours d'examen, \p HIWORD(wParam) = numro canal
	sn_info_stats,	//!< Message statistiques du signal
					//!< \p lParam = informations statistiques du signal
	sn_completed,	//!< Scan d'une frquence termin
					//!< (\p HIWORD(wParam) = numro canal, \p lParam = frquence)
	sn_info_nit,	//!< Message "lecture de la NIT"
	sn_info_clear,	//!< Effacement du message d'information
	sn_restore,		//!< Restauration des prfrences utilisateur
	sn_scan_end		//!< Recherche termine
					//!< (\p HIWORD(wParam) = tat 'ScanState'  la fin, \p lParam = nombre d'tapes)
};

/// Classe contenant les informations utilisateur mises en attente
/// en cas de recherche de chanes avec rcupration des prfrences utilisateur.
class CInfoUser : public vector<ChaineIdsUser>
{
public:
};

/// Donnes et mthodes ncessaires  la recherche de chanes
class ChannelsScanner : public WorkingThread
{
	enum SignalState {
		ss_error,			//!< Erreur  la dtection du signal
		ss_nosignal,		//!< Aucun signal dtect
		ss_unlocked,		//!< Signal dtect mais non verrouill
		ss_bad_quality,		//!< Signal dtect et verrouill, qualit insuffisante
		ss_signal_ok		//!< Signal apte  l'analyse
	};

	/// Dialogue recevant les messages de notification
	HWND				hwndDlg;

	/// Descripteur du fichier de log du scan
	FILE *				fLog;

	/// Informations rcupres dans la NIT pour dfinir les numros des chanes
	CInfoNIT			cInfoNIT;

	/// Accs aux informations utilisateurs pralablement rcupres
	const CInfoUser	&	cInfoUser;

	/// Liste fixe de 1000 index de descripteurs de chanes, pour chacun des 1000 numros
	/// de chane possibles (de 0  999 bien que le zro ne soit pas admis), utilise
	/// pour cartographier la numrotation et viter les doublons. Toute position
	/// correspondant  un numro non allou contient la valeur -1.
	SHORT				aRefNumeros[1000];

	// ### Mthodes

	/// Rinitialisation de la carte des chanes  vide
	void				raz_carte();

	/// Ralisation de la cartographie des chanes
	void				cartographier();

	/// Fonction d'excution principale effective du 'thread'.
	virtual DWORD		ThreadProc();

	/// Sortie d'un message de log
	void				log(LPCTSTR pszFmt, ...) const;

	/// Sortir les informations concernant un filtre
	void				logDeviceInfo(LPCTSTR pszTitle, LPCTSTR pszDevicePath);

	/**
	 * Fonction gnrant en mme temps le log des stats du signal  cet instant, et
	 * vrifiant si la qualit du signal est suffisante pour esprer recevoir quelque chose.
	 * \param[in]	nTuneDelay	Temps depuis lequel le tuner est syntonis (mS)
	 * \return		\p true si la qualit du signal est suffisante
	 **/
	SignalState			CheckQuality(UINT nTuneDelay) const;

	/**
	 * Attribution d'un nouveau numro pour une nouvelle chane,  partir d'un numro propos.
	 * Vrifie d'abord la disponibilit du numro propos, et le retourne s'il est disponible.
	 * S'il ne l'est pas, alors on cherche  transformer ce numro par dcalage, en ajoutant
	 * (ventuellement plusieurs fois) la valeur de la variable globale \p autonum_duplicate.
	 * S'il est impossible de trouver un numro selon ce critre (si tous les numros "dcals" possibles
	 * entre 1 et 999 sont pris), on retourne 0.
	 * \param[in]	nNumeroPropose	Numro propos pour la chane.
	 * \retval	Numro trouv (intervalle 1  999)
	 * \retval	0 si aucun numro dcal disponible
	 **/
	UINT16				nouveau_numero_decale(UINT16 nNumeroPropose) const;

	/**
	 * Attribution d'un nouveau numro pour une nouvelle chane qui n'a pas de numro propos.
	 * Cherche  attribuer ce numro parmi les numros disponibles, dans l'intervalle compris
	 * entre la valeur de la variable \p autonum_undef et 999, puis entre 1 et \p autonum_undef.
	 * \retval	Numro trouv (intervalle 1  999)
	 * \retval	0 si plus aucun numro disponible (hypothse assez improbable)
	 **/
	UINT16				nouveau_numero_chaine() const;

	/**
	 * Assignation d'un numro  une chane. Si une autre chane avait ce numro, les numros sont
	 * changs.
	 * \param[in]	ixChaine	Index de la chane dans la table
	 * \param[in]	nNumero		Numro  assigner
	 * \param[in]	bPref		\p true s'il s'agit d'une prfrence utilisateur (par opposition
	 *							 un numro automatiquement calcul)
	 **/
	void				assigne_numero(int ixChaine, UINT16 nNumero, bool bPref);

	/**
	 * Attribution d'un numro  toutes les chanes qui n'en ont pas reu.
	 * Si les chanes ont t tries auparavant, ce numro sera attribu selon l'ordre
	 * alphabtique de leur nom.
	 * Au passage, marque galement comme "inactives" les chanes qui ne diffusent pas
	 * couramment en norme MPEG2.
	 **/
	void				attribue_numeros();

	/**
	 * Restauration des prfrences utilisateur sauvegardes
	 **/
	void				restaure_prefs();

	/**
	 * Transfert des numros de chane de la NIT vers les chanes qui viennent d'tre ajoutes
	 * \retval	\p true si au moins une chane n'a pas reu de numro
	 **/
	bool				place_numeros();

	/**
	 * Syntonisation du tuner pour un canal, et valuation des caractristiques
	 * du signal prsent.
	 * \param[in]	nNoCanal	Numro du canal  explorer (en principe de 21  69)
	 * \param[out]	khz			Frquence de syntonisation
	 * \param[out]	sSStats		Statistiques de qualit du signal
	 * \return		Estimation du succs de la syntonisation
	 **/
	SignalState			tune(BYTE nNoCanal, INT32 & khz, SignalStatistics & sSStats);

	bool				parse(BYTE nNoCanal, INT32 khz);

public:
	/// Index du nombre de chanes lues  l'issue du scan des multiplex prcdents.
	/// L'intervalle entre cette valeur et la taille de la table des chanes permet de
	/// dfinir quelles sont les chanes qui viennent d'tre ajoutes dans le scan
	/// de multiplex le plus rcent.
	UINT				prev_size;

	ScanState			eScanState;
	int					nIndex;

	// ### Mthodes

	/// Liste de canaux (pour recherche de chanes)
	ListeCanaux			liste_scan;

	/// Constructeur
	ChannelsScanner(HWND hDlg, const CInfoUser & cInfoUsr);

	/// Destructeur
	~ChannelsScanner();
};
