/*
 * $Id: CmdSubst.cpp 293 2011-05-20 21:12:14Z gingko $
 * Copyright (C) 2011 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/** \file
 *	\brief Utilitaire de recherche et remplacement en ligne de commande.
 *
 *	Ce fichier permet de gnrer un utilitaire "CmdSubst.exe", qui s'utilise comme
 *  filtre en ligne de commande pour effectuer des recherches et remplacement entre
 *  l'entre standard et la sortie standard.
 *  Les oprations peuvent galement s'effectuer  l'aide d'expressions rgulires.
 **/

#include "stdafx.h"

#define HAVE_TR1 1
#define HAVE_BOOST 0

#if HAVE_TR1
	// La bibliothque TR1 avec le module <regex> est disponible nativement sous Visual Studio 2008,
	// Visual Studio 2010 et Visual C++ Express 2010.
	#include <regex>
	using namespace tr1;
#elif HAVE_BOOST
	// Les utilisateurs de Visual Studio 2005, Visual C++ Express 2005 et Visual C++ Express 2008 devront
	// utiliser la version de <regex> incluse dans la bibliothque open source 'boost' (ce qui requiert
	// de la tlcharger et de l'installer, et ensuite de compiler et de rendre accessible la partie
	// "regex" de cette bibliothque). Voir http://www.boost.org/.
	#include <boost/regex.hpp>
	using namespace boost;
#else
	#error Aucune bibliothque d'expressions rgulires disponible.
#endif

#ifdef _UNICODE
	#define tcin wcin
	#define tcout wcout
	#define tcerr wcerr
	#define tstring wstring
	#define tregex wregex
#else
	#define tcin cin
	#define tcout cout
	#define tcerr cerr
	#define tstring string
	#define tregex regex
#endif

void basic_subst(tstring & strBuffer, LPCTSTR pszSearch, LPCTSTR pszSubst)
{
	if (!pszSearch || *pszSearch == 0)
		return;

	tstring::size_type	nFound = 0;

	while ((nFound = strBuffer.find(pszSearch, nFound)) != strBuffer.npos) {
		tstring	strTmp;

		strTmp.assign(strBuffer.c_str(), nFound);
		strTmp += pszSubst;
		strTmp += strBuffer.substr(nFound + _tcslen(pszSearch));
		nFound +=_tcslen(pszSubst);

		if (strTmp == strBuffer)
			break;	// Pas question d'entrer dans une boucle infinie !

		swap(strTmp, strBuffer);
	}
}

void ApplyEscape(tstring & strBuffer)
{
	tstring::size_type	nFound;

	while ((nFound = strBuffer.find('\\')) != strBuffer.npos) {
		tstring	strTmp;
		LPTSTR	pszFound	 = &strBuffer[nFound];
		LPTSTR	pszNext	 = pszFound+1;
		TCHAR	szStr[2] = {0};

		if (*pszNext == 0)
			break;

		switch (*pszNext++) {

		case 'a':	szStr[0] = '\a'; break;
		case 'b':	szStr[0] = '\b'; break;
		case 'f':	szStr[0] = '\f'; break;
		case 'n':	szStr[0] = '\n'; break;
		case 'q':	szStr[0] = '"';  break;
		case 'r':	szStr[0] = '\r'; break;
		case 't':	szStr[0] = '\t'; break;
		case 'v':	szStr[0] = '\v'; break;

		case 'x':
			szStr[0] = _tcstoul(pszNext, &pszNext, 16);
			break;

		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
			szStr[0] = _tcstoul(pszNext-1, &pszNext, 8);
			break;

		default:
			szStr[0] = pszNext[-1];
		}

		strTmp.assign(strBuffer.c_str(), nFound);
		strTmp += szStr;
		strTmp += strBuffer.substr(nFound+(pszNext - pszFound));

		if (strTmp == strBuffer)
			break;	// Pas question d'entrer dans une boucle infinie !

		swap(strTmp, strBuffer);
	}
}

#define eAcute _T("\x82")
#define aGrave _T("\x85")
#define eGrave _T("\x8a")
#define iCirc  _T("\x8c")

struct OnePair
{
	tstring	strSearch;
	tstring	strSubst;
	bool	bRegExpr;

	OnePair() :
		bRegExpr(false)
	{
	}
};

int _tmain(int argc, _TCHAR* argv[])
{
	if (argc <= 1) {
		tcout
			<< _T("CmdSubst - v1.0 - filtre de recherche et remplacement.") << endl << endl
			<< _T("  Syntaxe :") << endl << endl
			<< _T("    CmdSubst [-r|-d] <rech1> [-d] <repl1> [ [-r|-d] <rechN> [-d] <replN> ...]") << endl << endl
			<< _T("      Chaque paire <rechN> <replN> est appliqu") eAcute _T("e tour ") aGrave _T(" tour ") aGrave _T(" chaque ligne du") << endl
			<< _T("      fichier soumis sur l'entr") eAcute _T("e standard, le premier ") eAcute _T("l") eAcute _T("ment ") eAcute _T("tant la cha") iCirc _T("ne") << endl
			<< _T("      recherch") eAcute _T("e, le second ") eAcute _T("tant la cha") iCirc _T("ne de remplacement. Les paires subs") eAcute _T("-") << endl
			<< _T("      quentes tiennent comptent ") eAcute _T("ventuellement  de la substitution d") eAcute _T("j") aGrave _T(" effec-") << endl
			<< _T("      tu") eAcute _T("e par les paires pr") eAcute _T("c") eAcute _T("dentes.") << endl << endl
			<< _T("  Options :") << endl << endl
			<< _T("   -r : pr") eAcute _T("c") eAcute _T("dant une paire,  indique  que celle-ci  sera trait") eAcute _T("e  en tant que") << endl
			<< _T("        deux  expressions  r") eAcute _T("guli") eGrave _T("res  respectant  la syntaxe ECMA.  Voir page") << endl
			<< _T("        web : http://msdn.microsoft.com/fr-fr/library/bb982727.aspx") << endl
			<< _T("   -d : pr") eAcute _T("c") eAcute _T("dant  la cha") iCirc _T("ne  recherch") eAcute _T("e  ou bien  la cha") iCirc _T("ne  de substitution,") << endl
			<< _T("        indique  que les caract") eGrave _T("res  d'") eAcute _T("chappement ('\\n', '\\t', etc)  y seront") << endl
			<< _T("        remplac") eAcute _T("s par les caract") eGrave _T("res qu'ils repr") eAcute _T("sentent.") << endl
			<< _T("        Notez  cependant que,  le traitement  se faisant  ligne par ligne,  le") << endl
			<< _T("        caract") eGrave _T("re d'") eAcute _T("chappement '\\n'  ne peut avoir de sens ") eAcute _T("ventuel  que dans") << endl
			<< _T("        la cha") iCirc _T("ne de substitution.") << endl
			<< _T("        Un code non standard y est aussi ajout") eAcute _T(" : '\\q' = '\"' (guillemet).") << endl << endl
			<< _T("  Le r") eAcute _T("sultat est ") eAcute _T("mis sur la sortie standard.") << endl;
		return 1;
	}

	vector<OnePair>	vPairs;
	int				nArg = 1;
	LPCTSTR			pszArg;

	while (nArg < argc) {
		OnePair		sPair;
		bool		bDecode = false;

		pszArg = argv[nArg++];

		if (_tcscmp(pszArg, _T("-r")) == 0) {
			sPair.bRegExpr = true;
		} else if (_tcscmp(pszArg, _T("-d")) == 0) {
			bDecode = true;
		}
		if (sPair.bRegExpr || bDecode) {
			if (nArg > argc) {
				tcerr << _T("## Expression de recherche manquante") << endl;
				return 1;
			}
			pszArg = argv[nArg++];
		}

		sPair.strSearch = pszArg;
		if (bDecode)
			ApplyEscape(sPair.strSearch);

		pszArg =  _T("");

		if (nArg < argc) {
			pszArg	= argv[nArg++];
			bDecode = _tcscmp(pszArg, _T("-d")) == 0;

			if (bDecode) {
				if (nArg > argc) {
					tcerr << _T("## Expression de substitution manquante") << endl;
					return 1;
				}
				pszArg = argv[nArg++];
			}
			sPair.strSubst = pszArg;
			if (bDecode)
				ApplyEscape(sPair.strSubst);
		}
		vPairs.push_back(sPair);
	}

	tstring	strBuffer;

	while (!tcin.eof()) {
		getline(tcin, strBuffer);

		for (vector<OnePair>::const_iterator it = vPairs.begin(); it != vPairs.end(); ++it) {
			if (it->bRegExpr) {
				try {
					tregex rx(it->strSearch);

					strBuffer = regex_replace(strBuffer, rx, it->strSubst);
				} catch (regex_error & rerr) {
					tcerr << _T("## Erreur expression r") eAcute _T("guli") eGrave _T("re, code ") << rerr.code() << endl;
					return 2;
				} catch (...) {
					tcerr << _T("## Exception inconnue") << endl;
					return 3;
				}
			} else {
				basic_subst(strBuffer, it->strSearch.c_str(), it->strSubst.c_str());
			}
		}

		if (tcin.eof())
			break;	// viter qu'une fin de ligne intempestive se rajoute  la fin du fichier
		tcout << strBuffer << endl;
	}

	if (!strBuffer.empty())
		tcout << strBuffer;

	return 0;
}
