/*
 * $Id: network.cpp 306 2011-09-28 09:28:04Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentation du filtre "Network provider" (interne).
 *
 *	Ce fichier dfinit l'interface du filtre (interne) "Network provider",
 *	qui est un pseudo-filtre de substitution au "Microsoft DVB-T Network Provider".
 *
 *	Il se place en tte de graphe, juste avant le tuner, et n'est utilis
 *	que dans le graphe du mode alternatif (le graphe normal utilisant effectivement
 *	le filtre "Microsoft DVB-T Network Provider").
 **/

#include "stdafx.h"
#include "network.h"

CPinBidon::CPinBidon(HRESULT *phr, CSource *pFilter) :
	CSourceStream(TEXT("Pin de sortie d'antenne"), phr, pFilter, L"Antenna out")
{
	myprintf(TEXT("CNetworkProvider : broche construite") EOL);
}


CPinBidon::~CPinBidon()
{
	myprintf(TEXT("CNetworkProvider : broche dtruite") EOL);

}

HRESULT CPinBidon::CheckMediaType(const CMediaType *pmt)
{
//	myprintf(TEXT("CNetworkProvider : CheckMediaType") EOL);
	if (*pmt->Type() == KSDATAFORMAT_TYPE_BDA_ANTENNA)
		return S_OK;
	return S_FALSE;
}

HRESULT CPinBidon::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	myprintf(TEXT("CNetworkProvider : GetMediaType %i") EOL, iPosition);
	if (iPosition == 0) {
		pMediaType->SetType(&KSDATAFORMAT_TYPE_BDA_ANTENNA);

		return S_OK;
	} else if (iPosition < 0) {
		return E_INVALIDARG;
	} else {
		return VFW_S_NO_MORE_ITEMS;
	}
}

HRESULT CPinBidon::FillBuffer(IMediaSample *pSamp) {
	// Note : il semble que des erreurs 0x80070001 soient produites  l'initialisation
	// dans CBaseFilter, en tentant d'exploiter les rsultats des premiers appels  cette
	// fonction. C'est vaguement gnant quoique pas catastrophique, nanmoins si un jour
	// on pouvait trouver un moyen lgant de traiter cette situation ...

/*
#ifdef USE_CONSOLE
	// Debugging - comptage "propre" (sans saut de ligne) du nombre de passages par cet endroit.
	static UINT nCount = 0;
	LPBYTE		ptr;

	pSamp->GetPointer(&ptr);
	myprintf(TEXT("CPinBidon FillBuffer (size=%u, ptr=0x%08x) - count = %u  \r"),
		pSamp->GetSize(), ptr, ++nCount);
#endif
*/
	// Dlai d'attente insr par prcaution : il a t constat que sur certaines versions
	// de Windows (en particulier sous Vista, quand on tourne en mode 32 bits dans un
	// Vista 64 bits), la boucle d'appel du "thread" dans lequel on se trouve demande
	// continuellement des donnes alors que dans les autres versions, elle s'arrte
	// d'elle-mme.
	// Le rsultat est une consommation de CPU considrable, comme si on se verrouillait
	// dans une boucle d'attente infinie.
	// L'insertion du dlai ne devrait pas poser de problmes, puisque de toutes faons
	// la broche de sortie n'a rien  fournir.
	Sleep(100);	// 100 millisecondes, a devrait tre pas trop mal.

	return S_OK;
}

HRESULT CPinBidon::DecideBufferSize(IMemAllocator * pAlloc,ALLOCATOR_PROPERTIES * ppropInputRequest)
{
	//myprintf(TEXT("CNetworkProvider : CPinBidon DecideBufferSize") EOL);
	return S_OK;
}

CNetworkProvider::CNetworkProvider(IUnknown *pUnk, HRESULT *phr) :
	CSource(TEXT(NETWORK_FILTER_NAME), NULL, __uuidof(this))
{
	HRESULT hr;
	m_bidon = new CPinBidon(&hr, this);

	if (phr) {
		if (m_bidon == NULL)
			*phr = E_OUTOFMEMORY;
		else {
			*phr = S_OK;
			myprintf(TEXT("CNetworkProvider : %") W2t TEXT(" construit") EOL, szFilterName);
		}
	}
}

CNetworkProvider::~CNetworkProvider()
{
	myprintf(TEXT("CNetworkProvider : %") W2t TEXT(" dtruit") EOL, szFilterName);
	delete m_bidon;
}

LPCWSTR CNetworkProvider::szFilterName = WTXT(NETWORK_FILTER_NAME);

CUnknown * WINAPI CNetworkProvider::CreateInstance(IUnknown *pUnk, HRESULT *phr) // static
{
	CNetworkProvider *pNewFilter = new CNetworkProvider(pUnk, phr);

	if (phr) {
		if (pNewFilter == NULL)
			*phr = E_OUTOFMEMORY;
		else
			*phr = S_OK;
	}

	return pNewFilter;
}

//
// GetPinCount
//
// Returns the number of pins this filter has
int CNetworkProvider::GetPinCount(void) {
	return 1;
}

//
// GetPin
//
// Return a non-addref'd pointer to pin n
// needed by CBaseFilter
CBasePin *CNetworkProvider::GetPin(int n) {
//	myprintf(TEXT("CNetworkProvider::GetPin %i") EOL, n);
	return (n==0) ? m_bidon : NULL;
}
